//#include <fstream.h>
//#include <iostream.h>
#include <stdio.h>
#include <string>
#include <math.h>
using namespace std;

#pragma warning (disable: 4996)

//---------------------------------------------------------------------------

#define pi 3.14159265358979323846264

//---------------------------------------------------------------------------

int main(int argc, char* argv[])
{
    if (argc < 4)
    {
        printf("Usage: cmod2raw <infile.txt> <outfile.raw> width height\n");
        return 1;
    }

    int width,height;
    sscanf(argv[3], "%d", &width);
    sscanf(argv[4], "%d", &height);
    width++;
    height++;
    printf("width  = %d\nheight = %d\n", width, height);

    double * dataD = new double [width*height];
    unsigned short * dataS = new unsigned short [width*height];
    memset(dataD, 0, width*height*sizeof(double));

    // Loading and converting data

    FILE * fin  = fopen(argv[1],"rb");
    if (fin==NULL)
    {
        printf("Can't open input file \"%s\"\n", argv[1]);
        return 2;
    }

    double xx,yy,zz;
    double nx,ny,nz;
    double tu,tv;
    double dist;
    double minDist = 1.0e+38;
    double maxDist = 0.0;
    double minTU = 1.0e+38;
    double maxTU = 0.0;
    double minTV = 1.0e+38;
    double maxTV = 0.0;
    int    c,wx,wy;
    int    i = 0;

    // find texture coordinates range
    for (;;)
    {
        c = fscanf(fin,"%lf %lf %lf %lf %lf %lf %lf %lf", &xx, &yy, &zz, &nx, &ny, &nz, &tu, &tv);
        if ((c==0)||(c==EOF)) break;
        if (minTU > tu) minTU = tu;
        if (maxTU < tu) maxTU = tu;
        if (minTV > tv) minTV = tv;
        if (maxTV < tv) maxTV = tv;
        i++;
    }

    printf("%d points\n", i);
    printf("min TU = %1.6f\n", minTU);
    printf("max TU = %1.6f\n", maxTU);
    printf("min TV = %1.6f\n", minTV);
    printf("max TV = %1.6f\n", maxTV);

    fseek(fin, 0, SEEK_SET);

    // read data
    for (;;)
    {
        c = fscanf(fin,"%lf %lf %lf %lf %lf %lf %lf %lf", &xx, &yy, &zz, &nx, &ny, &nz, &tu, &tv);
        if ((c==0)||(c==EOF)) break;

        xx = -xx;
        yy = -yy;
        dist = sqrt(xx*xx + yy*yy + zz*zz);

        wx = (int)fabs((tu-minTU)/(maxTU-minTU) * (width-1));
        wy = (int)fabs((tv-minTV)/(maxTV-minTV) * (height-1));
        //wy = (int)((tv+2.0) * (height-2));

        if ((wx<0)||(wx>=width)||(wy<0)||(wy>=height))
            printf("Vertex %d: index out of range (%d, %d)\n", i, wx, wy);
        else
        {
            dataD[wx + wy*width] = dist;
            if (minDist > dist) minDist = dist;
            if (maxDist < dist) maxDist = dist;
        }
    }

    fclose(fin);

    // Normalize and convert data

    printf("min dist = %1.6f\n", minDist);
    printf("max dist = %1.6f\n", maxDist);

    maxDist -= minDist;
    for (i=0; i<width*height; ++i)
        dataS[i] = (unsigned short)(255.0 * (dataD[i] - minDist) / maxDist);

    // Write data to out file

    FILE * fout = fopen(argv[2],"wb+");
    if (fout==NULL)
    {
        printf("Can't create output file \"%s\"\n", argv[2]);
        fclose(fout);
        return 3;
    }
    fwrite(dataS, width*height*sizeof(unsigned short), 1, fout);
    fclose(fout);

    delete [] dataS;
    delete [] dataD;

    return 0;
}

//---------------------------------------------------------------------------
